from django.conf import settings
from django.views.generic.detail import DetailView
from django.contrib.contenttypes.models import ContentType

from django_tables2 import RequestConfig

from apis_core.apis_entities.views import GenericListViewNew
from . forms2 import GenericRelationForm
from . rel_forms import PersonPlaceFilterFormHelper
from . tables import get_generic_relation_listview_table


class GenericRelationView(GenericListViewNew):

    # formhelper_class = GenericRelationForm
    context_filter_name = 'filter'
    paginate_by = 25
    template_name = getattr(
        settings,
        'APIS_LIST_VIEW_TEMPLATE',
        'apis_entities/generic_list.html'
    )
    login_url = '/accounts/login/'

    def get_table(self, **kwargs):
        relation = self.kwargs['entity'].lower()
        self.table_class = get_generic_relation_listview_table(relation)
        table = super(GenericListViewNew, self).get_table()
        RequestConfig(self.request, paginate={
            'page': 1, 'per_page': self.paginate_by}).configure(table)
        return table


class GenericRelationDetailView(DetailView):

    template_name = getattr(
        settings,
        'APIS_RELATIONS_DETAIL_VIEW_TEMPLATE',
        'apis_relations/relations_detail_generic.html'
    )

    def get_object(self):
        entity = self.kwargs['entity'].lower()
        instance = self.kwargs['pk'].lower()
        entity_model = ContentType.objects.get(
            app_label='apis_relations', model=entity).model_class()
        instance = entity_model.objects.get(pk=instance)
        return instance

    def get_context_data(self, **kwargs):
        context = super(GenericRelationDetailView, self).get_context_data()
        context['entity'] = self.kwargs['entity'].lower()
        context['entity_type'] = context['entity']
        return context



from crispy_forms.helper import FormHelper


class PersonPlaceFilterFormHelper(FormHelper):
    def __init__(self, *args, **kwargs):
        super(PersonPlaceFilterFormHelper, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.form_class = 'genericFilterForm'
        self.form_method = 'GET'
        self.helper.form_tag = False
        self.add_input(Submit('Filter', 'Search'))
        self.layout = Layout(
            Fieldset(
                'Basic search options',
                'related_person',
                'related_place',
                css_id="basic_search_fields"
                ),
            Accordion(
                AccordionGroup(
                    'Advanced search',
                    'relation_type',
                    css_id="more"
                    ),
                )
            )



from rest_framework import renderers
from apis_core.apis_tei.tei import TeiEntCreator


class EntityToTEI(renderers.BaseRenderer):

    media_type = "text/xml"
    format = 'tei'

    def render(self, data, media_type=None, renderer_context=None):
        tei_doc = TeiEntCreator(data)
        return tei_doc.serialize_full_doc()



from rest_framework import serializers
from apis_core.apis_vocabularies.serializers import TextTypeSerializer
from .models import Collection, Text, Source, Uri, TempEntityClass


class CollectionSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        #fields = '__all__'
        exclude = ('groups_allowed', )
        model = Collection


class TextSerializer(serializers.HyperlinkedModelSerializer):
    #kind = TextTypeSerializer()

    class Meta:
        fields = '__all__'
        model = Text


class SourceSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        fields = '__all__'
        model = Source


class UriSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        fields = '__all__'
        model = Uri


class TempEntityClassSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        fields = '__all__'
        model = TempEntityClass
